@@ -1,13 +1,12 @@
 from asyncio import AbstractEventLoop
-from typing import Callable, List, Optional, Union
+from typing import Any, Callable, List, Optional, Union
 
-from aiocli.commander import Application, ApplicationParser, ApplicationReturn, run_app
+from aiocli.commander import Application, ApplicationParser, run_app
 
 __all__ = (
     # commander
     'Application',
     'ApplicationParser',
-    'ApplicationReturn',
     # commander_app_wrappers
     'aws_run_app',
     'az_run_app',
@@ -25,8 +24,10 @@ def _cloud_run_app(
     argv: Optional[List[str]] = None,
     exit_code: bool = False,
     close_loop: bool = False,
-    parser: Optional[ApplicationParser] = None
-) -> ApplicationReturn:
+    parser: Optional[ApplicationParser] = None,
+    override_color: Optional[bool] = False,
+    override_return: Optional[bool] = True,
+) -> Any:
     return run_app(
         app=app,
         loop=loop,
@@ -35,6 +36,8 @@ def _cloud_run_app(
         exit_code=exit_code,
         close_loop=close_loop,
         parser=(lambda *args, **kwargs: argv) if parser is None else parser,
+        override_color=override_color,
+        override_return=override_return,
     )
 
 
