@@ -1,7 +1,16 @@
 import asyncio
+from asyncio import get_event_loop_policy
 from functools import partial
 from sys import version_info
-from typing import Any, Callable, Optional, Set
+from typing import Any, Callable, Coroutine, Optional, Set
+
+__all__ = (
+    # helpers
+    'all_tasks',
+    'iscoroutinefunction',
+    'resolve_function',
+    'resolve_coroutine',
+)
 
 
 def all_tasks(loop: Optional[asyncio.AbstractEventLoop] = None) -> Set["asyncio.Task[Any]"]:
@@ -27,3 +36,7 @@ async def resolve_function(func: Callable[..., Any], *args, **kwargs) -> Any:  #
     if iscoroutinefunction(func):
         return await func(*args, **kwargs)
     return func(*args, **kwargs)
+
+
+def resolve_coroutine(func: Callable[..., Coroutine[Any, Any, Any]], *args, **kwargs) -> Any:  # type: ignore
+    return get_event_loop_policy().get_event_loop().run_until_complete(func(*args, **kwargs))
